class Error (Exception): pass


class Snp:
    def __init__(self, line):
        #[P1] [SUB] [SUB]  [P2] [BUFF] [DIST] [LEN R] [LEN Q] [FRM]   [TAGS]
        #187  A     C      269  187    187    654     853     1       1   ref_name  qry_name
        try:
            l = line.rstrip().split('\t')
            self.ref_pos = int(l[0]) - 1
            self.ref_base = l[1]
            self.qry_base = l[2]
            self.qry_pos = int(l[3]) - 1
            self.ref_length = int(l[6])
            self.qry_length = int(l[7])
            self.ref_name = l[10]
            self.qry_name = l[11]
        except:
            raise Error('Error constructing pymummer.snp.Snp from mummer show-snps output at this line:\n' + line)


    def __eq__(self, other):
        return type(other) is type(self) and self.__dict__ == other.__dict__


    def __str__(self):
        return '\t'.join([str(x) for x in [
            self.ref_pos + 1,
            self.ref_base,
            self.qry_base,
            self.qry_pos + 1,
            self.ref_length,
            self.qry_length,
            self.ref_name,
            self.qry_name
        ]])





import os
import tempfile
import shutil
import pyfastaq
from pymummer import syscall

class Error (Exception): pass


class Runner:
    '''Handy reference for all the arguments needed for nucmer, delta-filter, show-coords, show-snps'''
    def __init__(
      self,
      ref,
      query,
      outfile,
      min_id=None,
      min_length=None,
      breaklen=None,
      coords_header=True,
      diagdiff=None,
      maxmatch=False,
      simplify=True,
      show_snps=False,
      snps_header=True,
      verbose=False,
      promer=False,
   ):
        self.qry = query
        self.ref = ref
        self.outfile = outfile
        self.min_id = min_id
        self.min_length = min_length
        self.breaklen = breaklen
        self.diagdiff = diagdiff
        self.coords_header = coords_header
        self.maxmatch = maxmatch
        self.simplify = simplify
        self.show_snps = show_snps
        self.snps_header = snps_header
        self.verbose = verbose
        self.use_promer = promer
    


    def _nucmer_command(self, ref, qry, outprefix):
        '''Construct the nucmer command'''
        if self.use_promer:
            command = 'promer'
        else:
            command = 'nucmer'
  
        command += ' -p ' + outprefix

        if self.breaklen is not None:
            command += ' -b ' + str(self.breaklen)

        if self.diagdiff is not None and not self.use_promer:
            command += ' -D ' + str(self.diagdiff)

        if self.maxmatch:
            command += ' --maxmatch'
            
        if not self.simplify and not self.use_promer:
        	command += ' --nosimplify'

        return command + ' ' + ref + ' ' + qry



    def _delta_filter_command(self, infile, outfile):
        '''Construct delta-filter command'''
        command = 'delta-filter'

        if self.min_id is not None:
            command += ' -i ' + str(self.min_id)

        if self.min_length is not None:
            command += ' -l ' + str(self.min_length)

        return command + ' ' + infile + ' > ' + outfile
        

    def _show_coords_command(self, infile, outfile):
        '''Construct show-coords command'''
        command = 'show-coords -dTlro'

        if not self.coords_header:
            command += ' -H'

        return command + ' ' + infile + ' > ' + outfile


    def _show_snps_command(self, infile, outfile):
        command = 'show-snps -TClr'

        if not self.snps_header:
            command += ' -H'

        return command + ' ' + infile + ' > ' + outfile


    def _write_script(self, script_name, ref, qry, outfile):
        '''Write commands into a bash script'''
        f = pyfastaq.utils.open_file_write(script_name)
        print(self._nucmer_command(ref, qry, 'p'), file=f)
        print(self._delta_filter_command('p.delta', 'p.delta.filter'), file=f)
        print(self._show_coords_command('p.delta.filter', outfile), file=f)
        if self.show_snps:
            print(self._show_snps_command('p.delta.filter', outfile + '.snps'), file=f)
        pyfastaq.utils.close(f)
    

    def run(self):
        '''
        Change to a temp directory
        Run bash script containing commands
        Place results in specified output file
        Clean up temp directory
        '''
        qry = os.path.abspath(self.qry)
        ref = os.path.abspath(self.ref)
        outfile = os.path.abspath(self.outfile)
        tmpdir = tempfile.mkdtemp(prefix='tmp.run_nucmer.', dir=os.getcwd())
        original_dir = os.getcwd()
        os.chdir(tmpdir)
        script = 'run_nucmer.sh'
        self._write_script(script, ref, qry, outfile) 
        syscall.run('bash ' + script, verbose=self.verbose)
        os.chdir(original_dir)
        shutil.rmtree(tmpdir)




import unittest
import os
from pymummer import snp_file, snp, variant

modules_dir = os.path.dirname(os.path.abspath(snp_file.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')

class TestUtils(unittest.TestCase):
    def test_snp_file(self):
        '''test coords_file'''
        expected = [
            '\t'.join(['133', 'G', '.', '122', '1', '122', '500', '489', '1', '1', 'ref', 'qry']),
            '\t'.join(['143', '.', 'C', '131', '1', '132', '500', '489', '1', '1', 'ref', 'qry']),
            '\t'.join(['253', 'T', 'A', '242', '120', '242', '500', '489', '1', '1', 'ref', 'qry'])
        ]

        expected = [snp.Snp(x) for x in expected]

        infiles = [os.path.join(data_dir, 'snp_file_test_with_header.snps'), os.path.join(data_dir, 'snp_file_test_no_header.snps')]

        for fname in infiles:
            fr = snp_file.reader(fname)
            snps = [x for x in fr]
            self.assertEqual(snps, expected)


    def test_get_all_variants(self):
        '''Test load all variants from file'''
        deletion_snps = [
            '\t'.join(['125', 'T', '.', '124', '1', '124', '500', '497', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['126', 'A', '.', '124', '1', '124', '500', '497', '1', '1', 'ref1', 'qry1']),
            '\t'.join(['127', 'C', '.', '124', '1', '124', '500', '497', '1', '1', 'ref1', 'qry1']),
        ]
        deletion_snps = [snp.Snp(x) for x in deletion_snps]
        deletion_variant = variant.Variant(deletion_snps[0])
        deletion_variant.update_indel(deletion_snps[1])
        deletion_variant.update_indel(deletion_snps[2])

        just_a_snp = '\t'.join(['386', 'C', 'T', '383', '115', '115', '500', '497', '1', '1', 'ref1', 'qry1'])
        snp_variant = variant.Variant(snp.Snp(just_a_snp))

        insertion_snps = [
            '\t'.join(['479', '.', 'G', '480', '0', '22', '500', '504', '1', '1', 'ref2', 'qry2']),
            '\t'.join(['479', '.', 'A', '481', '0', '22', '500', '504', '1', '1', 'ref2', 'qry2']),
            '\t'.join(['479', '.', 'T', '482', '0', '22', '500', '504', '1', '1', 'ref2', 'qry2']),
            '\t'.join(['479', '.', 'A', '483', '0', '22', '500', '504', '1', '1', 'ref2', 'qry2']),
        ]
        insertion_snps = [snp.Snp(x) for x in insertion_snps]
        insertion_variant = variant.Variant(insertion_snps[0])
        for i in range(1, len(insertion_snps)):
            insertion_variant.update_indel(insertion_snps[i])


        variants_from_file = snp_file.get_all_variants(os.path.join(data_dir, 'snp_file_test_get_all_variants.snps'))
        self.assertEqual(len(variants_from_file), 3)
        self.assertEqual(variants_from_file[0], deletion_variant)
        self.assertEqual(variants_from_file[1], snp_variant)
        self.assertEqual(variants_from_file[2], insertion_variant)



import unittest
import os
from pymummer import syscall


class TestSyscall(unittest.TestCase):
    def test_run_fail(self):
        '''Test that run raises error when command fails'''
        with self.assertRaises(syscall.Error):
            syscall.run('notacommandandthrowerror')


    def test_run_ok(self):
        '''Test run is ok on command that works'''
        syscall.run('ls')




import unittest
import os
import pyfastaq
from pymummer import alignment, snp, variant

modules_dir = os.path.dirname(os.path.abspath(alignment.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')

class TestNucmer(unittest.TestCase):
    def test_init_nucmer(self):
        '''test __init__ nucmer'''
        line = '\t'.join(['1', '100', '2', '200', '101', '202', '42.42', '123', '456', '-1', '0', 'ref', 'qry', '[FOO]'])
        a = alignment.Alignment(line)
        self.assertEqual(a.ref_start, 0)
        self.assertEqual(a.ref_end, 99)
        self.assertEqual(a.qry_start, 1)
        self.assertEqual(a.qry_end, 199)
        self.assertEqual(a.hit_length_ref, 101)
        self.assertEqual(a.hit_length_qry, 202)
        self.assertEqual(a.percent_identity, 42.42)
        self.assertEqual(a.ref_length, 123)
        self.assertEqual(a.qry_length, 456)
        self.assertEqual(a.frame, -1)
        self.assertEqual(a.ref_name, 'ref')
        self.assertEqual(a.qry_name, 'qry')


    def test_init_promer(self):
        '''test __init__ promer'''
        line = '\t'.join(['1', '1398', '4891054', '4892445', '1398', '1392', '89.55', '93.18', '0.21', '1398', '5349013', '1', '1', 'ref', 'qry', '[CONTAINED]'])

        a = alignment.Alignment(line)
        self.assertEqual(a.ref_start, 0)
        self.assertEqual(a.ref_end, 1397)
        self.assertEqual(a.qry_start, 4891053)
        self.assertEqual(a.qry_end, 4892444)
        self.assertEqual(a.hit_length_ref, 1398)
        self.assertEqual(a.hit_length_qry, 1392)
        self.assertEqual(a.percent_identity, 89.55)
        self.assertEqual(a.ref_length, 1398)
        self.assertEqual(a.qry_length, 5349013)
        self.assertEqual(a.frame, 1)
        self.assertEqual(a.ref_name, 'ref')
        self.assertEqual(a.qry_name, 'qry')


    def test_swap(self):
        ''' test swap'''
        l_in = ['1', '100', '2', '200', '101', '202', '42.42', '123', '456', '-1', '0', 'ref', 'qry']
        l_out = ['2', '200', '1', '100', '202', '101', '42.42', '456', '123', '-1', '0', 'qry', 'ref']
        a_in = alignment.Alignment('\t'.join(l_in))
        a_in._swap()
        self.assertEqual(a_in, alignment.Alignment('\t'.join(l_out)))
        a_in._swap()
        self.assertEqual(a_in, alignment.Alignment('\t'.join(l_in)))


    def test_qry_coords(self):
        '''Test qry_coords'''
        hits = ['\t'.join(['1', '100', '1', '100', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'qry']),
                '\t'.join(['1', '101', '100', '1', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'qry'])
        ]
        for h in hits:
            a = alignment.Alignment(h)
            self.assertEqual(pyfastaq.intervals.Interval(0,99), a.qry_coords())


    def test_ref_coords(self):
        '''Test ref_coords'''
        hits = ['\t'.join(['1', '100', '1', '100', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref']),
                '\t'.join(['100', '1', '100', '1', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref'])
        ]
        for h in hits:
            a = alignment.Alignment(h)
            self.assertEqual(pyfastaq.intervals.Interval(0,99), a.ref_coords())


    def test_on_same_strand(self):
        '''test on_same_strand'''
        self.assertTrue(alignment.Alignment('\t'.join(['1', '100', '1', '100', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref'])).on_same_strand())
        self.assertTrue(alignment.Alignment('\t'.join(['100', '1', '100', '1', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref'])).on_same_strand())
        self.assertFalse(alignment.Alignment('\t'.join(['1', '100', '100', '1', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref'])).on_same_strand())
        self.assertFalse(alignment.Alignment('\t'.join(['100', '1', '1', '100', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref'])).on_same_strand())


    def test_is_self_hit(self):
        '''Test is_self_hit'''
        tests = [('\t'.join(['1', '100', '1', '100', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref']), True),
            ('\t'.join(['1', '101', '1', '100', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref']), False),
            ('\t'.join(['2', '100', '1', '100', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref']), False),
            ('\t'.join(['1', '100', '1', '100', '100', '100', '100.00', '1000', '1000', '1', '1', 'ref', 'ref2']), False),
            ('\t'.join(['1', '100', '1', '100', '100', '100', '99.9', '1000', '1000', '1', '1', 'ref', 'ref']), False),
        ]

        for t in tests:
            a = alignment.Alignment(t[0])
            self.assertEqual(a.is_self_hit(), t[1])


    def test_reverse_reference(self):
        '''Test reverse_reference'''
        aln = alignment.Alignment('\t'.join(['100', '142', '1', '42', '43', '42', '100.00', '150', '100', '1', '1', 'ref', 'qry']))
        expected = alignment.Alignment('\t'.join(['51', '9', '1', '42', '43', '42', '100.00', '150', '100', '1', '1', 'ref', 'qry']))
        aln.reverse_reference()
        self.assertEqual(expected, aln)


    def test_reverse_query(self):
        '''Test reverse_query'''
        aln = alignment.Alignment('\t'.join(['100', '142', '1', '42', '43', '42', '100.00', '150', '100', '1', '1', 'ref', 'qry']))
        expected = alignment.Alignment('\t'.join(['100', '142', '100', '59', '43', '42', '100.00', '150', '100', '1', '1', 'ref', 'qry']))
        aln.reverse_query()
        self.assertEqual(expected, aln)


    def test_str(self):
        '''Test __str__'''
        l_in = ['1', '100', '2', '200', '101', '202', '42.42', '123', '456', '-1', '0', 'ref', 'qry']
        # the 10th column (counting from zero) is ignored and so not output by __str__
        l_out = l_in[:10] + l_in[11:]
        a = alignment.Alignment('\t'.join(l_in))
        self.assertEqual(str(a), '\t'.join(l_out))


    def test_to_msp_crunch(self):
        '''Test to_msp_crunch'''
        l_in = ['100', '110', '1', '10', '10', '11', '80.00', '123', '456', '-1', '0', 'ref', 'qry']
        a = alignment.Alignment('\t'.join(l_in))
        expected = '8 80.00 1 10 qry 100 110 ref'
        self.assertEqual(expected, a.to_msp_crunch())


    def test_qry_coords_from_ref_coord_test_bad_ref_coord(self):
        '''Test qry_coords_from_ref_coord with bad ref coords'''
        aln = alignment.Alignment('\t'.join(['100', '200', '1', '100', '100', '100', '100.00', '300', '300', '1', '1', 'ref', 'qry']))
        with self.assertRaises(alignment.Error):
            got = aln.qry_coords_from_ref_coord(98, [])

        with self.assertRaises(alignment.Error):
            got = aln.qry_coords_from_ref_coord(200, [])


    def test_qry_coords_from_ref_coord_test_same_strand(self):
        '''Test qry_coords_from_ref_coord on same strand'''
        aln = alignment.Alignment('\t'.join(['100', '200', '1', '101', '100', '100', '100.00', '300', '300', '1', '1', 'ref', 'qry']))
        snp0 = snp.Snp('\t'.join(['140', 'A', 'T', '40', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # snp
        snp0 = variant.Variant(snp0)
        snp1 = snp.Snp('\t'.join(['140', 'A', '.', '40', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from qry
        snp2 = snp.Snp('\t'.join(['141', 'C', '.', '40', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from qry
        del1 = variant.Variant(snp1)
        del2 = variant.Variant(snp1)
        self.assertTrue(del2.update_indel(snp2))
        snp3 = snp.Snp('\t'.join(['150', '.', 'A', '50', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from ref
        snp4 = snp.Snp('\t'.join(['150', '.', 'C', '51', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from ref
        snp5 = snp.Snp('\t'.join(['150', '.', 'G', '52', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from ref
        ins1 = variant.Variant(snp3)
        ins2 = variant.Variant(snp3)
        self.assertTrue(ins2.update_indel(snp4))
        self.assertTrue(ins2.update_indel(snp5))

        tests = [
            (99, [], (0, False)),
            (100, [], (1, False)),
            (199, [], (100, False)),
            (119, [del1], (20, False)),
            (149, [], (50, False)),
            (149, [del1], (49, False)),
            (149, [del2], (48, False)),
            (159, [], (60, False)),
            (159, [ins1], (61, False)),
            (159, [ins2], (63, False)),
            (159, [del1, ins1], (60, False)),
            (159, [del1, ins2], (62, False)),
            (159, [del2, ins1], (59, False)),
            (159, [del2, ins2], (61, False)),
            (139, [del1], (39, True)),
            (139, [snp0], (40, False)),
            (149, [ins1], (49, True)),
        ]

        for ref_coord, variant_list, expected in tests:
            got = aln.qry_coords_from_ref_coord(ref_coord, variant_list)
            self.assertEqual(expected, got)
            # if we reverse the direction of hit in query and reference, should get the same answer
            aln.qry_start, aln.qry_end = aln.qry_end, aln.qry_start
            aln.ref_start, aln.ref_end = aln.ref_end, aln.ref_start
            got = aln.qry_coords_from_ref_coord(ref_coord, variant_list)
            self.assertEqual(expected, got)
            aln.qry_start, aln.qry_end = aln.qry_end, aln.qry_start
            aln.ref_start, aln.ref_end = aln.ref_end, aln.ref_start


    def test_qry_coords_from_ref_coord_test_different_strand(self):
        '''Test qry_coords_from_ref_coord on different strand'''
        aln = alignment.Alignment('\t'.join(['100', '200', '101', '1', '100', '100', '100.00', '300', '300', '1', '1', 'ref', 'qry']))
        snp0 = snp.Snp('\t'.join(['140', 'A', 'T', '40', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # snp
        snp0 = variant.Variant(snp0)
        snp1 = snp.Snp('\t'.join(['140', 'A', '.', '40', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from qry
        snp2 = snp.Snp('\t'.join(['141', 'C', '.', '40', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from qry
        del1 = variant.Variant(snp1)
        del2 = variant.Variant(snp1)
        self.assertTrue(del2.update_indel(snp2))
        snp3 = snp.Snp('\t'.join(['150', '.', 'A', '50', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from ref
        snp4 = snp.Snp('\t'.join(['150', '.', 'C', '51', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from ref
        snp5 = snp.Snp('\t'.join(['150', '.', 'G', '52', 'x', 'x', '300', '300', 'x', 'x', 'ref', 'qry'])) # del from ref
        ins1 = variant.Variant(snp3)
        ins2 = variant.Variant(snp3)
        self.assertTrue(ins2.update_indel(snp4))
        self.assertTrue(ins2.update_indel(snp5))

        tests = [
            (99, [], (100, False)),
            (100, [], (99, False)),
            (199, [], (0, False)),
            (119, [], (80, False)),
            (119, [del1], (80, False)),
            (149, [], (50, False)),
            (149, [del1], (51, False)),
            (149, [del2], (52, False)),
            (159, [], (40, False)),
            (159, [ins1], (39, False)),
            (159, [ins2], (37, False)),
            (159, [del1, ins1], (40, False)),
            (159, [del1, ins2], (38, False)),
            (159, [del2, ins1], (41, False)),
            (159, [del2, ins2], (39, False)),
            (139, [del1], (39, True)),
            (139, [snp0], (60, False)),
            (149, [ins1], (49, True)),
        ]

        for ref_coord, variant_list, expected in tests:
            got = aln.qry_coords_from_ref_coord(ref_coord, variant_list)
            self.assertEqual(expected, got)
            # if we reverse the direction of hit in query and reference, should get the same answer
            aln.qry_start, aln.qry_end = aln.qry_end, aln.qry_start
            aln.ref_start, aln.ref_end = aln.ref_end, aln.ref_start
            got = aln.qry_coords_from_ref_coord(ref_coord, variant_list)
            self.assertEqual(expected, got)
            aln.qry_start, aln.qry_end = aln.qry_end, aln.qry_start
            aln.ref_start, aln.ref_end = aln.ref_end, aln.ref_start





import os
import glob
import shutil
import sys
from setuptools import setup, find_packages


required_progs = ['nucmer', 'show-coords', 'show-snps', 'delta-filter']
found_all_progs = True
print('Checking MUMmer programs found in path:')

for program in required_progs:
    if shutil.which(program) is None:
        found_all_progs = False
        found = '   NOT FOUND'
    else:
        found = '          OK'

    print(found, program, sep='\t')


if not found_all_progs:
    print('Cannot install because some programs from the MUMer package not found.', file=sys.stderr)
    sys.exit(1)


setup(
    name='pymummer',
    version='0.7.0',
    description='Wrapper for MUMmer',
    packages = find_packages(),
    author='Martin Hunt, Nishadi De Silva',
    author_email='path-help@sanger.ac.uk',
    url='https://github.com/sanger-pathogens/pymummer',
    test_suite='nose.collector',
    install_requires=['pyfastaq >= 3.10.0'],
    tests_require=['nose >= 1.3'],
    license='GPLv3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    ],
)



import unittest
import os
from pymummer import snp


modules_dir = os.path.dirname(os.path.abspath(snp.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestSnp(unittest.TestCase):
    def test_str(self):
        '''Test __str__'''
        l_in = ['187', 'A', 'C', '269', '187', '187', '654', '853', '1', '1', 'ref_name', 'qry_name']
        # only use columns 0-3, 6-7, 10-11
        l_out = l_in[:4] + l_in[6:8] + l_in[10:]
        s = snp.Snp('\t'.join(l_in))
        self.assertEqual(str(s), '\t'.join(l_out))



import unittest
import os
import filecmp
from pymummer import nucmer

modules_dir = os.path.dirname(os.path.abspath(nucmer.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestRunner(unittest.TestCase):
    def test_nucmer_command(self):
        '''test _nucmer_command'''
        tests = [
            [nucmer.Runner('ref', 'qry', 'outfile'), 'nucmer -p pre ref qry'],
            [nucmer.Runner('ref', 'qry', 'outfile', breaklen=42), 'nucmer -p pre -b 42 ref qry'],
            [nucmer.Runner('ref', 'qry', 'outfile', diagdiff=11), 'nucmer -p pre -D 11 ref qry'],
            [nucmer.Runner('ref', 'qry', 'outfile', diagdiff=11, promer=True), 'promer -p pre ref qry'],
            [nucmer.Runner('ref', 'qry', 'outfile', maxmatch=True), 'nucmer -p pre --maxmatch ref qry'],
            [nucmer.Runner('ref', 'qry', 'outfile', simplify=False), 'nucmer -p pre --nosimplify ref qry'],
            [nucmer.Runner('ref', 'qry', 'outfile', promer=True), 'promer -p pre ref qry'],
            [nucmer.Runner('ref', 'qry', 'outfile', promer=True, breaklen=42), 'promer -p pre -b 42 ref qry'],
            [nucmer.Runner('ref', 'qry', 'outfile', promer=True, maxmatch=True), 'promer -p pre --maxmatch ref qry'],
            [nucmer.Runner('ref', 'qry', 'outfile', promer=True, simplify=False), 'promer -p pre ref qry']
        ]

        for l in tests:
            self.assertEqual(l[0]._nucmer_command('ref', 'qry', 'pre'), l[1])


    def test_delta_filter_command(self):
        '''test _delta_filter_command'''
        tests = [
            [nucmer.Runner('ref', 'qry', 'outfile'), 'delta-filter infile > outfile'],
            [nucmer.Runner('ref', 'qry', 'outfile', min_id=42), 'delta-filter -i 42 infile > outfile'],
            [nucmer.Runner('ref', 'qry', 'outfile', min_length=43), 'delta-filter -l 43 infile > outfile'],
        ]

        for l in tests:
            self.assertEqual(l[0]._delta_filter_command('infile', 'outfile'), l[1])
        

    def test_show_coords_command(self):
        '''test _show_coords_command'''
        tests = [
            [nucmer.Runner('ref', 'qry', 'outfile', coords_header=False), 'show-coords -dTlro -H infile > outfile'],
            [nucmer.Runner('ref', 'qry', 'outfile'), 'show-coords -dTlro infile > outfile']
        ]

        for l in tests:
            self.assertEqual(l[0]._show_coords_command('infile', 'outfile'), l[1])


    def test_show_snps_command(self):
        '''test _show_snps_command'''
        tests = [
            [nucmer.Runner('ref', 'qry', 'outfile', snps_header=False), 'show-snps -TClr -H infile > outfile'],
            [nucmer.Runner('ref', 'qry', 'outfile'), 'show-snps -TClr infile > outfile']
        ]

        for l in tests:
            self.assertEqual(l[0]._show_snps_command('infile', 'outfile'), l[1])


    def test_write_script_no_snps(self):
       '''test _write_script no snps'''
       tmp_script = 'tmp.script.sh'
       r = nucmer.Runner('ref', 'qry', 'outfile')
       r._write_script(tmp_script, 'ref', 'qry', 'outfile')
       expected = os.path.join(data_dir, 'nucmer_test_write_script_no_snps.sh')
       self.assertTrue(filecmp.cmp(expected, tmp_script, shallow=False))
       os.unlink(tmp_script)


    def test_write_script_with_snps(self):
       '''test _write_script with snps'''
       tmp_script = 'tmp.script.sh'
       r = nucmer.Runner('ref', 'qry', 'outfile', show_snps='outfile.snps')
       r._write_script(tmp_script, 'ref', 'qry', 'outfile')
       expected = os.path.join(data_dir, 'nucmer_test_write_script_with_snps.sh')
       self.assertTrue(filecmp.cmp(expected, tmp_script, shallow=False))
       os.unlink(tmp_script)


    def test_run_nucmer(self):
        '''test run_nucmer'''
        qry = os.path.join(data_dir, 'nucmer_test_qry.fa')
        ref = os.path.join(data_dir, 'nucmer_test_ref.fa')
        tmp_out = 'tmp.nucmer.out'
        runner = nucmer.Runner(ref, qry, tmp_out, coords_header=False, show_snps=True, snps_header=False)
        runner.run()
        expected = os.path.join(data_dir, 'nucmer_test_out.coords')
        self.assertTrue(filecmp.cmp(tmp_out, expected, shallow=False))
        self.assertTrue(filecmp.cmp(tmp_out + '.snps', expected + '.snps', shallow=False))
        os.unlink(tmp_out)
        os.unlink(tmp_out + '.snps')
        

